<?php

namespace Fufle\ORM\Database;

use PDO;
use Fufle\ORM\Database\QueryData;
use Fufle\ORM\Database\Connectors\IConnector;

class ConnectionManager {
    private static $connection;
    private static $connectors = [];
    private static string $connectionString;
    private static Credentials $credentials;

    public static function RegisterConnector(IConnector $connector) {
        self::$connectors[] = $connector;
    }

    public static function Connect(string $driver, $host, $port = null, $database = null, $user = null, $password = null) {
        foreach (self::$connectors as $c) {
            if ($c->driverEquals($driver)) {
                $credentials = new Credentials($host, $port, $database, $user, $password);
                self::$connectionString = $c->getConnectionString($credentials);
                self::ConnectWithString(self::$connectionString);
                return;
            }
        }
        throw new Exception('Driver not found');
    }

    public static function ConnectWithString(string $connectionString) {
        self::$connection = new PDO($connectionString);
    }

    public static function Query(string $query, array|object $params = [], string $output = 'stdObject') {
        $data = new QueryData($query, $params);
        $data->normalize();
        $stmt = self::$connection->prepare($data->query);
        $stmt->execute($data->params);
        $stmt->setFetchMode(PDO::FETCH_CLASS, $output);
        return $stmt->fetch();
    }

    public static function QueryAll(string $query, array|object $params = [], string $output = 'stdObject') {
        $data = new QueryData($query, $params);
        $data->normalize();
        $stmt = self::$connection->prepare($data->query);
        $stmt->execute($data->params);
        $stmt->setFetchMode(PDO::FETCH_CLASS, $output);
        return $stmt->fetchAll();
    }
}